/* 
 * Eneso GTK+ Decoration Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <cairo.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "edecor_private.h"
#include "edecor.h"

enum
{
	TOP_LEFT = 0,
	TOP,
	TOP_RIGHT,
	RIGHT,
	BOTTOM_RIGHT,
	BOTTOM,
	BOTTOM_LEFT,
	LEFT
};

typedef struct _EDecorWindow EDecorWindow;
struct _EDecorWindow
{
	GtkWidget*	widget;
	gint 		highlighted_button;
	gint		width, height;
};

typedef struct
{
	gint x, y, width, height;
}
WidgetPosition;

typedef struct
{
	gboolean highlighted;
	gboolean disabled;
	gboolean pressed;
}
WidgetStatus;

/* Constant numbers for the window */
static const gint x_title_bar_button [2] = { 30, 53 };
static const gint y_title_bar_button = 6;

static const gint border_width = 10;

/* Images */
static struct
{
	cairo_surface_t* title_bar_focused;
	cairo_surface_t* title_bar_unfocused;
	cairo_surface_t* button [2];
	cairo_surface_t* highlighted_button [2];
}
images;

static GdkCursor* cursor_resize [8];

static void
window_handle_widgets	(GtkWidget* widgets);
static void
handle_widget 			(GtkWidget* widget, gpointer data);
static void 
get_widget_position 	(GtkWidget* widget, WidgetPosition* position);
static void 
get_widget_status 		(GtkWidget* widget, WidgetStatus* status);

/* Callbacks */
static gboolean
window_expose_event_cb 		(GtkWidget *widget, GdkEventExpose *event, gpointer data);
static gboolean
window_focus_event_cb 		(GtkWidget *widget, GdkEventFocus *event, gpointer data);
static gboolean
window_motion_event_cb 		(GtkWidget *widget, GdkEventMotion *event, gpointer data);
static gboolean
window_button_cb		 	(GtkWidget *widget, GdkEventButton *event, gpointer data);
static gboolean
window_destroy_event_cb		(GtkWidget* widget, GdkEvent* event, gpointer data);

static gboolean
button_expose_event_cb		(GtkWidget *widget, GdkEventExpose *event, gpointer data);
static gboolean
checkb_expose_event_cb		(GtkWidget *widget, GdkEventExpose *event, gpointer data);
static gboolean
radiob_expose_event_cb		(GtkWidget *widget, GdkEventExpose *event, gpointer data);
static gboolean
combobox_expose_event_cb	(GtkWidget *widget, GdkEventExpose *event, gpointer data);
static gboolean
frame_expose_event_cb 		(GtkWidget *widget, GdkEventExpose *event, gpointer data);


	void
edecor_init (gchar* image_folder)
{
	gchar*	filename;

	/* Load the images */
#define LOAD_IMAGE(image,file)	\
	filename = g_build_filename (image_folder, file, NULL);	\
	images.image = cairo_image_surface_create_from_png (filename);	\
	g_free (filename);

	LOAD_IMAGE (title_bar_focused, "titlebar_focused.png");
	LOAD_IMAGE (title_bar_unfocused, "titlebar_unfocused.png");
#ifndef __MACH__
	LOAD_IMAGE (button [0], "titlebar_close.png");
	LOAD_IMAGE (button [1], "titlebar_minimise.png");
	LOAD_IMAGE (highlighted_button [0], "titlebar_close_highlighted.png");
	LOAD_IMAGE (highlighted_button [1], "titlebar_minimise_highlighted.png");

	/* Create the resize cursor */
	cursor_resize [TOP_LEFT] =
		gdk_cursor_new (GDK_TOP_LEFT_CORNER);
	cursor_resize [TOP] =
		gdk_cursor_new (GDK_TOP_SIDE);
	cursor_resize [TOP_RIGHT] =
		gdk_cursor_new (GDK_TOP_RIGHT_CORNER);
	cursor_resize [RIGHT] =
		gdk_cursor_new (GDK_RIGHT_SIDE);
	cursor_resize [BOTTOM_RIGHT] =
		gdk_cursor_new (GDK_BOTTOM_RIGHT_CORNER);
	cursor_resize [BOTTOM] =
		gdk_cursor_new (GDK_BOTTOM_SIDE);
	cursor_resize [BOTTOM_LEFT] =
		gdk_cursor_new (GDK_BOTTOM_LEFT_CORNER);
	cursor_resize [LEFT] =
		gdk_cursor_new (GDK_LEFT_SIDE);
#endif	/* __MACH__ */

#undef LOAD_IMAGE
}

	void
edecor_terminate (void)
{
#define FREE_IMAGE(var) \
	cairo_surface_destroy (images.var);

	FREE_IMAGE (title_bar_focused);
	FREE_IMAGE (title_bar_unfocused);
#ifndef __MACH__
	FREE_IMAGE (button [0]);
	FREE_IMAGE (button [1]);
	FREE_IMAGE (highlighted_button [0]);
	FREE_IMAGE (highlighted_button [1]);
#endif	/* __MACH__ */

#undef FREE_IMAGE
}

	void
edecor_decorate_window (GtkWidget* window, GtkWidget* vbox)
{
	EDecorWindow*	edecor_window;
	gint			width, height;
	GtkWidget*		alignment_titlebar_spacing;

	/* Allocate memory for the structure */
	edecor_window = g_malloc (sizeof (EDecorWindow));

	edecor_window->widget = window;
	edecor_window->highlighted_button = -1;

	edecor_window->width = edecor_window->height = -1;

#ifndef __MACH__
	/* Connect the signals */
	gtk_widget_add_events (window, 
			GDK_STRUCTURE_MASK		|
			GDK_BUTTON_PRESS_MASK	|
			GDK_BUTTON_RELEASE_MASK |
			GDK_POINTER_MOTION_MASK);	

	g_signal_connect (window,
			"motion-notify-event", G_CALLBACK (window_motion_event_cb), edecor_window);	
	g_signal_connect (window, 
			"button-press-event", G_CALLBACK (window_button_cb), edecor_window);	
	g_signal_connect (window, 
			"button-release-event", G_CALLBACK (window_button_cb), edecor_window);

	/* We will draw the decoration */
	gtk_window_set_decorated (GTK_WINDOW (window), FALSE);

	/* Add spacing for the title bar */
	alignment_titlebar_spacing = gtk_alignment_new (0, 0, 1, 1);
	gtk_widget_set_size_request (alignment_titlebar_spacing, -1, 27);
	gtk_box_pack_start (GTK_BOX (vbox), alignment_titlebar_spacing, FALSE, FALSE, 0);
	gtk_box_reorder_child (GTK_BOX (vbox), alignment_titlebar_spacing, 0);
	gtk_widget_show (alignment_titlebar_spacing);

#endif	/* __MACH__ */

	g_signal_connect (window, 
			"focus-in-event", G_CALLBACK (window_focus_event_cb), edecor_window);
	g_signal_connect (window, 
			"focus-out-event", G_CALLBACK (window_focus_event_cb), edecor_window);
	g_signal_connect (window, 
			"expose-event", G_CALLBACK (window_expose_event_cb), edecor_window);
	g_signal_connect (window, 
			"destroy-event", G_CALLBACK (window_destroy_event_cb), edecor_window);
	
	gtk_widget_set_app_paintable (window, TRUE);

	/* Handle the widgets */
	window_handle_widgets (window);
}

	void
edecor_draw_window_title_bar (cairo_t* cr, gint width, gint height, gboolean active)
{
	gint	i;

	cairo_rectangle (cr, 0, 0, width, height);
	cairo_clip (cr);

	for (i = 0; i < width; i += 6)
	{
		cairo_set_source_surface (cr,
				active ? images.title_bar_focused : images.title_bar_unfocused, i, 0);
		cairo_paint (cr);
	}

	cairo_reset_clip (cr);
}

	void
edecor_draw_window_background (cairo_t* cr, gint x, gint y, gint width, gint height)
{
	cairo_rectangle (cr, x, y, width, height);

	cairo_set_source_rgb (cr, 0.95, 0.95, 0.95);

	cairo_fill (cr);
}

	static void
window_handle_widgets (GtkWidget* widget)
{
	gtk_container_foreach (GTK_CONTAINER (widget), handle_widget, NULL);
}

	static void
handle_widget (GtkWidget* widget, gpointer data)
{
	gint	i;

	/* If its type is one we can draw, mark it as application paintable and
	 * connect to the expose signal */
	if (GTK_IS_RADIO_BUTTON (widget))
	{
		g_signal_connect (widget, "expose-event",
				G_CALLBACK (radiob_expose_event_cb), NULL);
		gtk_widget_set_app_paintable (widget, TRUE);
	}
	else if (GTK_IS_CHECK_BUTTON (widget))
	{
		g_signal_connect (widget, "expose-event",
				G_CALLBACK (checkb_expose_event_cb), NULL);
		gtk_widget_set_app_paintable (widget, TRUE);
	}
	else if (GTK_IS_BUTTON (widget))
	{
		g_signal_connect (widget, "expose-event",
				G_CALLBACK (button_expose_event_cb), NULL);
		gtk_widget_set_app_paintable (widget, TRUE);
	}
	else if (GTK_IS_COMBO_BOX (widget))
	{
		g_signal_connect (widget, "expose-event",
				G_CALLBACK (combobox_expose_event_cb), NULL);
		gtk_widget_set_app_paintable (widget, TRUE);
	}
	else if (GTK_IS_FRAME (widget))
	{
		g_signal_connect (widget, "expose-event",
				G_CALLBACK (frame_expose_event_cb), NULL);
		gtk_widget_set_app_paintable (widget, TRUE);
	}

	/* If it is a container check its children */
	if (GTK_IS_CONTAINER (widget))
	{
		gtk_container_foreach (GTK_CONTAINER (widget), handle_widget, NULL);
	}

	/* If it is a notebook, the widgets that are used as tab labels must be
	 * looked for by hand */
	if (GTK_IS_NOTEBOOK (widget))
	{
		for (i = 0; i < gtk_notebook_get_n_pages (GTK_NOTEBOOK (widget)); i++)
		{
			GtkWidget*	page;
			GtkWidget*	label;

			page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (widget), i);
			label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (widget), page);

			if (label)
				handle_widget (label, NULL);
		}
	}

	gtk_widget_show_all (widget);
}

	static void
get_widget_position (GtkWidget* widget, WidgetPosition* position)
{
	position->x = widget->allocation.x;
	position->y = widget->allocation.y;
	position->width = widget->allocation.width;
	position->height = widget->allocation.height;
}

	static void
get_widget_status (GtkWidget* widget, WidgetStatus* status)
{
	status->highlighted = (GTK_WIDGET_STATE (widget) == GTK_STATE_PRELIGHT);
	status->disabled = !GTK_WIDGET_SENSITIVE (widget);
	status->pressed = (GTK_WIDGET_STATE (widget) == GTK_STATE_ACTIVE);

	if (GTK_IS_TOGGLE_BUTTON (widget))
		status->pressed |= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
}

	static gboolean
window_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	EDecorWindow*	edecor_window;	
	cairo_t*		cr;
	gboolean		active;
	gint			width, height;
	gint			i;

	edecor_window = (EDecorWindow*) data;

	/* Get the window size and status */
	gtk_window_get_size (GTK_WINDOW (widget), &width, &height);
	active = gtk_window_is_active (GTK_WINDOW (widget));

	cr = gdk_cairo_create (widget->window);

	/* Background */
	edecor_draw_window_background (cr, 0, 0, width, height);

#ifndef __MACH__
	/* Title bar */
	edecor_draw_window_title_bar (cr, width, TITLE_BAR_HEIGHT, active);

	/* Window title */
	cairo_set_source_rgb (cr, 1, 1, 1);
	edecor_draw_text (cr, width >> 1, 16, gtk_window_get_title (GTK_WINDOW (widget)),
			"Times New Roman 13px", TRUE);

	/* Title bar buttons */
	if (gtk_window_get_deletable (GTK_WINDOW (widget)))
	{
		for (i = 0; i < 2; i++)
		{	
			cairo_set_source_surface (cr, 
					edecor_window->highlighted_button == i ?
					images.highlighted_button [i] :
					images.button [i], 
					width - x_title_bar_button [i], y_title_bar_button);
			cairo_paint (cr);
		}
	}

	/* Window border */
	if (active)
		cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
	else
		cairo_set_source_rgb (cr, 0.7, 0.7, 0.7);

	cairo_set_line_width (cr, 1.0);
	
	cairo_rectangle (cr, 0.5, 0.5, width - 1, height - 1);
	cairo_stroke (cr);

#endif	/* __MACH__ */

	cairo_destroy (cr);

	return FALSE;
}

	static gboolean
window_focus_event_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	/* If the window gets or loses focus repaint it,
	 * because the title bar changes */
	gtk_widget_queue_draw (widget);

	return FALSE;
}

	static gboolean
button_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t*			cr;
	WidgetPosition		pos;
	WidgetStatus		status;
	gint				text_width, text_height;
	PangoLayout*		layout; 

	/*
	 * GtkButton
	 */

	cr = gdk_cairo_create (widget->window);
	get_widget_position (widget, &pos);
	get_widget_status (widget, &status);

	/* Background */
	if (!status.disabled)
	{
		if (status.pressed)
			cairo_set_source_rgb (cr, 0.9, 0.9, 0.9);
		else if (status.highlighted)
			cairo_set_source_rgb (cr, 0.99, 0.99, 0.99);
		else
			cairo_set_source_rgb (cr, 0.97, 0.97, 0.97);

		edecor_draw_round_rect (cr, pos.x, pos.y, pos.width, pos.height, BUTTON_BORDER_RADIUS);
		cairo_fill (cr);
	}

	/* Border */
	cairo_set_line_width (cr, 1.0);

	if (status.highlighted)
		cairo_set_source_rgb (cr, 0.75, 0.75, 0.75);
	else if (status.disabled)
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.9);
	else 
		cairo_set_source_rgb (cr, 0.7, 0.7, 0.7);

	cairo_rectangle (cr, pos.x + 0.5, pos.y + 0.5, pos.width - 1, pos.height - 1);

	cairo_stroke (cr);

	/* Text */
	layout = gtk_widget_create_pango_layout (widget, 
			gtk_button_get_label (GTK_BUTTON (widget)));

	pango_layout_get_size (layout, &text_width, &text_height);

	gtk_paint_layout (widget->style, widget->window, 
			GTK_WIDGET_STATE (widget), FALSE, NULL, widget, "label", 
			pos.x + (pos.width >> 1) - ((float) text_width / PANGO_SCALE) / 2, 
			pos.y + (pos.height >> 1) +
			(status.pressed ? 1 : 0) - ((float) text_height / PANGO_SCALE) / 2,
			layout);

	g_object_unref (layout);


	cairo_destroy (cr);

	return TRUE;
}

	static gboolean
checkb_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t* 			cr;
	WidgetPosition 		pos;
	WidgetStatus		status;
	gint 				text_width, text_height;
	PangoLayout*		layout; 
	gint 				y;

	cr = gdk_cairo_create (widget->window);
	get_widget_position (widget, &pos);
	get_widget_status (widget, &status);

	y = pos.y + ((pos.height - CHECK_BUTTON_WIDTH) >> 1);

	/* Border */
	cairo_set_line_width (cr, 0.5);

	if (status.highlighted)
		cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
	else if (status.disabled)
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.9);
	else
		cairo_set_source_rgb (cr, 0.4, 0.4, 0.4);

	cairo_set_line_width (cr, 1.0);
	cairo_rectangle (cr, pos.x + 0.5, y + 0.5, CHECK_BUTTON_WIDTH, CHECK_BUTTON_WIDTH);
	cairo_stroke (cr);

	/* Pressed background */
	if (status.pressed && !status.disabled)
	{
		cairo_set_source_rgb (cr, 0.57, 0.6, 0.65);

		cairo_set_line_width (cr, 2.0);

		cairo_move_to (cr, pos.x + 4, y + 6);
		cairo_line_to (cr, pos.x + 6, y + 10);
		cairo_line_to (cr, pos.x + CHECK_BUTTON_WIDTH - 4, y + 4);
		cairo_stroke (cr);
	}

	/* Text */
	layout = gtk_widget_create_pango_layout (widget, 
			gtk_button_get_label (GTK_BUTTON (widget)));

	pango_layout_get_size (layout, &text_width, &text_height);

	gtk_paint_layout (widget->style, widget->window, 
			GTK_WIDGET_STATE (widget), FALSE, NULL, widget, "label", 
			pos.x + CHECK_BUTTON_WIDTH + 5, 
			pos.y + (pos.height >> 1) - ((float) text_height / PANGO_SCALE) / 2,
			layout);

	g_object_unref (layout);

	cairo_destroy (cr);

	return TRUE;
}

	static gboolean
radiob_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t* 			cr;
	WidgetPosition 		pos;
	WidgetStatus		status;
	gint 				text_width, text_height;
	PangoLayout*		layout; 
	gint 				xc, yc;

	cr = gdk_cairo_create (widget->window);
	get_widget_position (widget, &pos);
	get_widget_status (widget, &status);

	xc = pos.x + RADIO_BUTTON_RADIUS;
	yc = pos.y + (pos.height >> 1);

	/* Border */
	cairo_set_line_width (cr, 1.0);

	if (status.highlighted)
		cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
	else if (status.disabled)
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.9);
	else
		cairo_set_source_rgb (cr, 0.4, 0.4, 0.4);

	cairo_arc (cr, xc, yc, RADIO_BUTTON_RADIUS, 0, 2.0 * M_PI);
	cairo_stroke (cr);

	/* Pressed background */
	if (status.pressed && !status.disabled)
	{
		cairo_set_source_rgb (cr, 0.57, 0.6, 0.65);

		cairo_arc (cr, xc, yc, RADIO_BUTTON_RADIUS - 4, 0, 2.0 * M_PI);
		cairo_fill (cr);
	}

	/* Text */
	layout = gtk_widget_create_pango_layout (widget, 
			gtk_button_get_label (GTK_BUTTON (widget)));

	pango_layout_get_size (layout, &text_width, &text_height);

	gtk_paint_layout (widget->style, widget->window, 
			GTK_WIDGET_STATE (widget), FALSE, NULL, widget, "label", 
			pos.x + (RADIO_BUTTON_RADIUS << 1) + 5, 
			pos.y + (pos.height >> 1) - ((float) text_height / PANGO_SCALE) / 2,
			layout);

	g_object_unref (layout);

	cairo_destroy (cr);

	return TRUE;
}

	static gboolean
combobox_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t*		cr;
	WidgetPosition	pos;
	WidgetStatus	status;
	gint			text_width, text_height;
	PangoLayout*	layout; 

	cr = gdk_cairo_create (widget->window);
	get_widget_position (widget, &pos);
	get_widget_status (widget, &status);

	/* Background */
	cairo_save (cr);

	cairo_rectangle (cr, pos.x, pos.y, pos.width, pos.height);
	cairo_clip (cr);

	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);

	cairo_rectangle (cr, pos.x, pos.y, pos.width - 1, pos.height);
	cairo_fill (cr);

	cairo_restore (cr);

	/* Border */
	if (status.disabled)
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.9);
	else
		cairo_set_source_rgb (cr, 0.4, 0.4, 0.4);

	cairo_set_line_width (cr, 0.5);
	cairo_rectangle (cr, pos.x, pos.y, pos.width - 1, pos.height);
	cairo_stroke (cr);

	/* Triangle */
	cairo_move_to (cr, pos.x + pos.width - 19, pos.y + 7);
	cairo_line_to (cr, pos.x + pos.width - 9, pos.y + 7);
	cairo_line_to (cr, pos.x + pos.width - 14, pos.y + 16);
	cairo_close_path (cr);

	if (status.disabled)
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.9);
	else
		cairo_set_source_rgb (cr, 0.4, 0.4, 0.4);

	cairo_fill (cr);

	/* Text */
	layout = gtk_widget_create_pango_layout (widget, 
			gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget)));

	pango_layout_get_size (layout, &text_width, &text_height);

	gtk_paint_layout (widget->style, widget->window, 
			GTK_WIDGET_STATE (widget), FALSE, NULL, widget, "label", 
			pos.x + 5, 
			pos.y + (pos.height >> 1) - ((float) text_height / PANGO_SCALE) / 2,
			layout);

	g_object_unref (layout);

	cairo_destroy (cr);

	return TRUE;
}

	static gboolean
frame_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t*		cr;
	WidgetPosition	pos;
	WidgetStatus	status;

	cr = gdk_cairo_create (widget->window);
	get_widget_position (widget, &pos);
	get_widget_status (widget, &status);

	/* Background */
	if (status.disabled)
		cairo_set_source_rgb (cr, 0.9, 0.9, 0.9);
	else
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);

	cairo_rectangle (cr, pos.x, pos.y, pos.width, pos.height);
	cairo_fill (cr);

	/* Border */
	if (status.disabled)
		cairo_set_source_rgb (cr, 0.8, 0.8, 0.9);
	else
		cairo_set_source_rgb (cr, 0.4, 0.4, 0.4);

	cairo_set_line_width (cr, 1.0);
	cairo_rectangle (cr, pos.x + 0.5, pos.y + 0.5, pos.width - 1, pos.height - 1);
	cairo_stroke (cr);

	cairo_destroy (cr);

	return FALSE;
}

	static gboolean
window_motion_event_cb (GtkWidget *widget, GdkEventMotion *event, gpointer data)
{
	EDecorWindow*	edecor_window;
	gint			width, height;
	gint			x, y, i;
	gint			xv, yv;
	GdkCursor*		cursor;

	edecor_window = (EDecorWindow*) data;

	gtk_window_get_size (GTK_WINDOW (widget), &width, &height);
	gtk_window_get_position (GTK_WINDOW (widget), &xv, &yv);

	gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL); 
	x -= xv;
	y -= yv;

	/* Check whether it was over a title bar button */
	if (gtk_window_get_deletable (GTK_WINDOW (widget)))
	{
		for (i = 0; i < 2; i++)
		{
			if (x >= width - x_title_bar_button [i] && 
					x <= width - x_title_bar_button [i] + TITLE_BAR_BUTTON_WIDTH &&
					y >= y_title_bar_button &&
					y <= y_title_bar_button + TITLE_BAR_BUTTON_HEIGHT)
			{
				edecor_window->highlighted_button = i;
				gtk_widget_queue_draw (widget);

				return FALSE;
			}
		}

		if (edecor_window->highlighted_button >= 0)
			gtk_widget_queue_draw (widget);
		edecor_window->highlighted_button = -1;
	}

	return FALSE;
}

	static gboolean
window_button_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	EDecorWindow*	edecor_window;
	gint			width, height;
	gint			x, y, type, button;
	gint			i;

	edecor_window = (EDecorWindow*) data;

	gtk_window_get_size (GTK_WINDOW (widget), &width, &height);

	x = event->x;
	y = event->y;
	type = event->type;
	button = event->button;

	if (button == 1)
	{
		/* Check whether it was over a title bar button */
		if (gtk_window_get_deletable (GTK_WINDOW (widget)))
		{
			for (i = 0; i < 2; i++)
			{
				if (x >= width - x_title_bar_button [i] &&
						x <= width - x_title_bar_button [i] + TITLE_BAR_BUTTON_WIDTH &&
						y >= y_title_bar_button &&
						y <= y_title_bar_button + TITLE_BAR_BUTTON_HEIGHT)
				{
					if (type == GDK_BUTTON_RELEASE)
					{
						switch (i)
						{
							case 0:
								gtk_widget_hide (widget);
								break;

							case 1:
								gtk_window_iconify (GTK_WINDOW (widget));
								break;
						}
					}

					edecor_window->highlighted_button = -1;
					gtk_widget_queue_draw (widget);

					return FALSE;
				}
			}
		}

		if (type == GDK_BUTTON_PRESS)
		{
			if (y < TITLE_BAR_HEIGHT) 
			{
				gtk_window_begin_move_drag (GTK_WINDOW (widget),
						event->button, event->x_root, event->y_root, event->time);
			}
		}
	}

	return FALSE;
}

	static gboolean
window_destroy_event_cb (GtkWidget* widget, GdkEvent* event, gpointer data)
{
	EDecorWindow* edecor_window;

	edecor_window = (EDecorWindow*) data;

	if (edecor_window)
		g_free (edecor_window);
}
