/* 
 * enPathia 
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib/gi18n.h>
#include <cairo.h>
#include <gtk/gtk.h>
#include <pango/pangocairo.h>
#include "edecor.h"
#include "misc.h"
#include "profile.h"
#include "help.h"

#define N_PAGES	11

const gchar*
help_shortcut_title [N_HELP_SHORTCUTS] =
{
	N_("Introduction"),
	N_("Configuration"),
	N_("Calibration")
};

static const gint
page_shortcut [N_HELP_SHORTCUTS] =
{
	0, 4, 9
};

static struct
{
	GtkWidget*	window_help;
	GtkWidget*	vbox_help;
	GtkWidget*	drawingarea;
	GtkWidget*	button_previous;
	GtkWidget*	button_next;
}
widgets;

static gchar* 			page_title [N_PAGES];
static gchar* 			page_content [N_PAGES];
static cairo_surface_t*	page_image [N_PAGES];

static cairo_surface_t* 	img_enpathia;
static cairo_pattern_t* 	pat;

static gint current_page;

static void
load_text		(gchar* filename);
static cairo_surface_t*
load_help_image	(gchar* filename);
static void
change_page		(gint page);


	void 
create_help (void)
{
	GtkBuilder*	builder;
	gchar*		filename;
	gchar*		data_file;

	/* Load the interface file */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/help.glade");
	
	if (!gtk_builder_add_from_file (builder, data_file, NULL))
		g_warning ("Could not load the help interface");

	g_free (data_file);

	/* Get the widgets */
#define GET_WIDGET(widget,name) \
	widgets.widget = GTK_WIDGET (gtk_builder_get_object (builder, name)); \
	if (widgets.widget == NULL) g_warning ("Widget not found: %s", name);

	GET_WIDGET (window_help, "window_help");
	GET_WIDGET (vbox_help, "vbox_help");
	GET_WIDGET (drawingarea, "drawingarea");
	GET_WIDGET (button_previous, "button_previous");
	GET_WIDGET (button_next, "button_next");

#undef GET_WIDGET

	gtk_builder_connect_signals (builder, &widgets);
	g_object_unref (G_OBJECT (builder));

	gtk_widget_set_sensitive (widgets.button_previous, FALSE);

	/* Load the text and images */
	filename = g_build_filename ("/help/", language_name [current_language], "help", NULL);
	data_file = get_data_file (filename);

	load_text (data_file);

	g_free (data_file);
	g_free (filename);

	data_file = get_data_file ("/images/enpathia.png");
	img_enpathia = cairo_image_surface_create_from_png (data_file);
	g_free (data_file);

	page_image [0] = load_help_image ("eneso.png");
	page_image [1] = load_help_image ("enpathia.png");
	page_image [2] = load_help_image ("move.png");
	page_image [3] = load_help_image ("menu.png");
	page_image [4] = load_help_image ("window.png");
	page_image [5] = load_help_image ("profiles.png");
	page_image [6] = load_help_image ("configuration_movement.png");
	page_image [7] = load_help_image ("configuration_switches.png");
	page_image [8] = load_help_image ("unlock.png");
	page_image [9] = load_help_image ("calibration_1.png");
	page_image [10] = load_help_image ("calibration_2.png");

	edecor_decorate_window (widgets.window_help, widgets.vbox_help);
}

	void 
show_help (gint shortcut)
{
	change_page (page_shortcut [shortcut]);

	gtk_window_present (GTK_WINDOW (widgets.window_help));
}

	void
destroy_help (void)
{
	gint i;

	cairo_surface_destroy (img_enpathia);

	for (i = 0; i < N_PAGES; i++)
		cairo_surface_destroy (page_image [i]);
}

	G_MODULE_EXPORT gboolean
drawingarea_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	cairo_t*	cr;
	gint		width, height;

	cr = gdk_cairo_create (widget->window);

	width	= widget->allocation.width;
	height	= widget->allocation.height;

	/* Background */
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_set_source_rgb (cr, 1, 1, 1);
	cairo_fill (cr);

	/* Title box */
	cairo_set_source_rgb (cr, 0.85, 0.85, 0.85);
	edecor_draw_round_rect (cr, width - 300, 25, 310, 30, 10);
	cairo_fill (cr);

	cairo_set_source_rgb (cr, 0.02, 0.17, 0.28);
	edecor_draw_text (cr, width - 145, 40, page_title [current_page],
			FONT " Bold 14px", TRUE);

	/* Border */
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_set_source_rgb (cr, 0.33, 0.5, 0.52);
	cairo_set_line_width (cr, 1.0);
	cairo_stroke (cr);

	/* Logo */
	cairo_set_source_surface (cr, img_enpathia, 20, 20);
	cairo_paint (cr);

	/* Content */
	cairo_set_source_rgb (cr, 0, 0, 0);
	edecor_draw_text_multiline (cr, 30, (height >> 1) + 30, page_content [current_page], FONT " 13px",
			width - 250, TRUE);

	/* Image */
	cairo_set_source_surface (cr, page_image [current_page], 
			width - 220 + ((220 - cairo_image_surface_get_width (page_image [current_page])) >> 1),
			90 + ((height - 150 - cairo_image_surface_get_height (page_image [current_page])) >> 1));

	cairo_paint (cr);
}

	G_MODULE_EXPORT void
button_quit_clicked_cb (GtkWidget *widget, gpointer data)
{
	gtk_widget_hide (widgets.window_help);
}

	G_MODULE_EXPORT void
button_previous_clicked_cb (GtkWidget *widget, gpointer data)
{
	change_page (current_page - 1);
}
	G_MODULE_EXPORT void
button_next_clicked_cb (GtkWidget *widget, gpointer data)
{
	change_page (current_page + 1);
}

	static void
load_text (gchar* filename)
{
	gchar*	contents;
	gint	i;

	/* Initialize */
	for (i = 0; i < N_PAGES; i++)
	{
		page_title [i] = NULL;
		page_content [i] = NULL;
	}

	/* Load the file */
	if (!g_file_get_contents (filename, &contents, NULL, NULL))
		return;

	/* Tokenize the sections */
	strtok (contents, "#\r\n");

	for (i = 0; i < N_PAGES; i++)
	{
		page_title [i] = strtok (NULL, "#\r\n");
		page_content [i] = strtok (NULL, "#");
	}
}

	static cairo_surface_t*
load_help_image (gchar* filename)
{
	cairo_surface_t*	surface;
	gchar*				file;
	gchar*				image_file;

	file = g_build_filename ("/help/", language_name [current_language], filename, NULL);
	image_file = get_data_file (file);

	surface = cairo_image_surface_create_from_png (image_file);

	g_free (image_file);
	g_free (file);

	return surface;
}

	static void
change_page (gint page)
{
	if (page < 0)
		page = 0;
	else if (page >= N_PAGES)
		page = N_PAGES - 1;

	current_page = page;

	gtk_widget_set_sensitive (widgets.button_previous, current_page > 0);
	gtk_widget_set_sensitive (widgets.button_next, current_page < N_PAGES - 1);

	gtk_widget_queue_draw (widgets.window_help);
}
