/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>
#include <glib/gi18n.h>
#include <glib/gstdio.h>

#ifdef G_OS_WIN32
#include <windows.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "edecor.h"
#include "misc.h"

#ifdef __MACH__
	GtkOSXApplication*	gtk_osx_app	= NULL;
#endif	/* __MACH__ */

gint	screen_width;
gint	screen_height;

static gboolean	waiting	= FALSE;

static gchar*	data_dir;
static gchar*	config_dir;

static gint
dialog	(gchar* title, gchar* message, gchar* label_button1, gchar* label_button2);

static gboolean
wait_cb			(gpointer data);
static gboolean
destroy_widget	(gpointer data);

	void
wait_ms (guint time)
{
	waiting = TRUE;
	g_timeout_add (time, wait_cb, NULL);

	while (waiting)
		gtk_main_iteration ();
}

	void
find_data_dir (void)
{
#ifdef G_OS_WIN32
	gchar*	win32_dir;

	win32_dir = g_win32_get_package_installation_directory_of_module (NULL);

	data_dir = g_build_filename (win32_dir, "share", "enpathia", NULL);
		
	if (!g_file_test (data_dir, G_FILE_TEST_EXISTS))
	{
		g_free (data_dir);
		data_dir = NULL;
	}

	g_free (win32_dir);
#else
	const gchar* const* data_dirs;
	gint i;
	
	data_dir = NULL;

	data_dirs = g_get_system_data_dirs ();
	
	for (i = 0; data_dirs [i] != NULL; i++)
	{
		data_dir = g_build_filename (data_dirs [i], "enpathia", NULL);
		
		if (g_file_test (data_dir, G_FILE_TEST_EXISTS))
			break;

		g_free (data_dir);
		data_dir = NULL;
	}
#endif

	if (data_dir == NULL)
		data_dir = g_strdup ("../data/");
}

	void
find_config_dir (void)
{
	const gchar* dir;
	gchar*	profiles_dir;

	dir = g_get_user_config_dir ();
	config_dir = g_build_filename (dir, "enpathia", NULL);
		
	profiles_dir = g_build_filename (config_dir, "profiles", NULL);
		
	if (!g_file_test (profiles_dir, G_FILE_TEST_EXISTS))
		g_mkdir_with_parents (profiles_dir, 0777);

	g_free (profiles_dir);
}

	gchar*
get_data_file (gchar* filename)
{
	return g_build_filename (data_dir, filename, NULL);
}

	gchar*
get_config_file (gchar* filename)
{
	return g_build_filename (config_dir, filename, NULL);
}

	void
execute_uri (gchar* uri)
{
#ifdef G_OS_WIN32
	ShellExecute (NULL, NULL, uri, NULL, NULL, SW_SHOWDEFAULT);
#else
#ifdef __MACH__
	gchar*	command;

	command = g_strdup_printf ("open %s", uri);

	system (command);

	g_free (command);
#else
	gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL);
#endif
#endif
}

	gint
copy_file (gchar* from, gchar* to)
{
	FILE*	f_from;
	FILE*	f_to;
	gchar*	buffer;
	size_t	size;

	/* Open both files */
	f_from = (FILE*) g_fopen (from, "rb");
	if (f_from == NULL)
	{
		g_warning (_("Could not open %s"), from);
		return -1;
	}

	f_to = (FILE*) g_fopen (to, "wb");
	if (f_to == NULL)
	{
		g_warning (_("Could not open %s"), to);
		fclose (f_from);
		return -1;
	}

	/* Create a buffer as long as the file */
	fseek (f_from, 0, SEEK_END);
	size = ftell (f_from);
	rewind (f_from);

	/* Read from the source file and write to the target */
	buffer = g_malloc (size);
	if (buffer != NULL)
	{
		fread (buffer, 1, size, f_from);
		fwrite (buffer, 1, size, f_to);

		g_free (buffer);
	}

	fclose (f_from);
	fclose (f_to);

	return 0;
}

	void
get_screen_size (void)
{
	GdkScreen* screen;

	screen = gdk_screen_get_default ();

	screen_width = gdk_screen_get_width (screen); 
	screen_height = gdk_screen_get_height (screen);
}

	void
center_pointer (void)
{
	gdk_display_warp_pointer (gdk_display_get_default (),
			gdk_screen_get_default (),
			screen_width >> 1,
#ifndef __MACH__
			screen_height >> 1
#else
			/* This is quite ugly, but as hide_pointer() doesn't work under Mac
			 * we need to move the pointer a little bit so it doesn't interfere
			 * with the window's content */
			(screen_height >> 1) + 20
#endif	/* __MACH__ */
			);
}

	void
hide_pointer (GtkWidget* widget)
{
#ifndef __MACH__
	static gchar pixels [] = { 0 };
	GdkCursor*	cursor;
	GdkPixmap*	image; 
	GdkPixmap*	mask; 
	GdkColor	color = { 0, 0, 0, 0 };

	g_return_if_fail (GTK_IS_WIDGET (widget));

	image = gdk_bitmap_create_from_data (NULL, pixels, 1, 1);
	mask = gdk_bitmap_create_from_data (NULL, pixels, 1, 1);

	cursor = gdk_cursor_new_from_pixmap (image, mask, &color, &color, 0, 0);

	gdk_pixmap_unref (image);
	gdk_pixmap_unref (mask);

	gtk_widget_realize (widget);
	gdk_window_set_cursor (widget->window, cursor);

	gdk_cursor_destroy (cursor);
#endif	/* __MACH__ */
}

	void
set_pointer (GtkWidget* widget, GdkCursorType type)
{
	GdkCursor*	cursor;

	cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
			type);
	gdk_window_set_cursor (widget->window, cursor);

	gdk_cursor_destroy (cursor);
}

	void
set_env_var (gchar* name, gchar* value)
{
	gchar	text [50];

	sprintf (text, "%s=%s", name, value);
	putenv (g_strdup (text));	
}

	void
get_env_var (gchar* name, gchar* value)
{
	gchar*	v;

	v = getenv (name);

	if (v == NULL)
		strcpy (value, "");
	else
		strcpy (value, v);
}

	void
dialog_info (gchar* title, gchar* message)
{
	dialog (title, message, _("Close"), NULL);
}

	gboolean
dialog_yes_no (gchar* title, gchar* message)
{
	return (dialog (title, message, _("Yes"), _("No")) == 1);
}

	void
dialog_transient (gchar* message, gchar* image_file)
{
	GtkBuilder*	builder;
	GtkWidget*	dialog_transient;
	GtkWidget*	vbox_transient;
	GtkWidget*	image;
	GtkWidget*	label_message;
	gchar*		data_file;
	gchar		text [200];

	/* Load the interface */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/transient.glade");

	if (!gtk_builder_add_from_file (builder, data_file, NULL))
		g_warning ("Could not load the transient dialog interface");

	g_free (data_file);

	/* Get the widgets */
	dialog_transient = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_transient"));
	vbox_transient = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_transient"));
	image = GTK_WIDGET (gtk_builder_get_object (builder, "image"));
	label_message = GTK_WIDGET (gtk_builder_get_object (builder, "label_message"));

	gtk_builder_connect_signals (builder, NULL);
	g_object_unref (G_OBJECT (builder));

	edecor_decorate_window (dialog_transient, vbox_transient);

	gtk_window_set_keep_above (GTK_WINDOW (dialog_transient), TRUE);

	/* Init the widgets */
	data_file = get_data_file (image_file);
	gtk_image_set_from_file (GTK_IMAGE (image), data_file);
	g_free (data_file);

	sprintf (text, "<span font=\"" FONT " 13px\" foreground=\"#333\">%s</span>", message);
	gtk_label_set_markup (GTK_LABEL (label_message), text);

	/* Show the dialog */
	gtk_widget_show (dialog_transient);

#ifdef __MACH__
	gtk_window_set_keep_above (GTK_WINDOW (dialog_transient), TRUE);
#endif	/* __MACH__ */

	g_timeout_add_seconds (2, destroy_widget, dialog_transient);
}

	gboolean
file_chooser (gchar** filename, gchar* title, gboolean open, gchar* default_name,
		gchar* filter_name, gchar* filter_ext)
{
	GtkWidget*		dialog;
	GtkFileFilter*	filter;
	gboolean		result = FALSE;

	/* Create the dialog */
	dialog = gtk_file_chooser_dialog_new (title, NULL,
			open ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			open ? GTK_STOCK_OPEN : GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);

	if (default_name)
		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), default_name);

	/* Create the filter */
	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, filter_name);
	gtk_file_filter_add_pattern (filter, filter_ext);
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);

	/* Show the dialog */
	result = (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT);
	if (result == TRUE)
		*filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

	gtk_widget_destroy (dialog);

	return result;
}

	static gint
dialog (gchar* title, gchar* message, gchar* label_button1, gchar* label_button2)
{
	GtkBuilder*	builder;
	GtkWidget*	dialog_message;
	GtkWidget*	vbox_message;
	GtkWidget*	image;
	GtkWidget*	label_title;
	GtkWidget*	label_message;
	GtkWidget*	button1;
	GtkWidget*	button2;
	gchar*		data_file;
	gchar		text [500];
	gint		response;

	/* Load the interface */
	builder = gtk_builder_new ();

	data_file = get_data_file ("interface/message.glade");

	if (!gtk_builder_add_from_file (builder, data_file, NULL))
		g_warning ("Could not load the message interface");

	g_free (data_file);

	/* Get the widgets */
	dialog_message = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_message"));
	vbox_message = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_message"));
	image = GTK_WIDGET (gtk_builder_get_object (builder, "image"));
	label_title = GTK_WIDGET (gtk_builder_get_object (builder, "label_title"));
	label_message = GTK_WIDGET (gtk_builder_get_object (builder, "label_message"));
	button1 = GTK_WIDGET (gtk_builder_get_object (builder, "button1"));
	button2 = GTK_WIDGET (gtk_builder_get_object (builder, "button2"));

	gtk_builder_connect_signals (builder, NULL);
	g_object_unref (G_OBJECT (builder));

	edecor_decorate_window (dialog_message, vbox_message);

	gtk_window_set_keep_above (GTK_WINDOW (dialog_message), TRUE);

	/* Init the widgets */
	data_file = get_data_file ("/images/enabled.png");
	gtk_image_set_from_file (GTK_IMAGE (image), data_file);
	g_free (data_file);

	if (label_button1)
		gtk_button_set_label (GTK_BUTTON (button1), label_button1);
	else
		gtk_widget_hide (button1);

	if (label_button2)
		gtk_button_set_label (GTK_BUTTON (button2), label_button2);
	else
		gtk_widget_hide (button2);

	if (title)
	{
		sprintf (text, "<span font=\"" FONT " Bold 14px\" foreground=\"#042b47\">%s</span>", title);
		gtk_label_set_markup (GTK_LABEL (label_title), text);
	}
	else
		gtk_widget_hide (label_title);

	if (message)
	{
		sprintf (text, "<span font=\"" FONT " 13px\" foreground=\"#333\">%s</span>", message);
		gtk_label_set_markup (GTK_LABEL (label_message), text);
	}
	else
		gtk_widget_hide (label_message);

#ifdef __MACH__
	gtk_widget_show (dialog_message);
	gtk_window_set_keep_above (GTK_WINDOW (dialog_message), TRUE);
#endif	/* __MACH__ */

	response = gtk_dialog_run (GTK_DIALOG (dialog_message));

	gtk_widget_destroy (dialog_message);

	return response;
}

	static gboolean
wait_cb (gpointer data)
{
	waiting = FALSE;

	return FALSE;
}

	static gboolean
destroy_widget (gpointer data)
{
	gtk_widget_destroy (GTK_WIDGET (data));

	return FALSE;
}
